from pystrictconfig.core import List, Integer, Map, Enum, OneOf, Any, Float, String


def test_list1():
    schema = List()

    assert schema.validate([])


def test_list2():
    schema = List(strict=False)

    assert schema.validate([1, 2, 3])


def test_list3():
    schema = List(data_type=Integer())

    assert schema.validate([1, 2, 3])


def test_list4():
    schema = List(data_type=Integer())

    assert not schema.validate([1.0, 2.0, 3.0])


def test_list5():
    schema = List(data_type=Integer(strict=False))

    assert schema.validate([1.0, 2.0, 3.0])


def test_list6():
    schema = List(data_type=Integer())

    assert schema.get([1.0, 2.0, 3.0]) == [1, 2, 3]


def test_list7():
    schema = List(data_type=Integer())

    assert not schema.validate(1)


def test_map1():
    schema = Map()

    assert schema.validate({})


def test_map2():
    schema = Map(strict=False)

    assert schema.validate({1: 2})


def test_map3():
    schema = Map()

    assert not schema.validate({1: 2})


def test_map4():
    schema = Map(schema={'nest1': Integer()})

    assert not schema.validate({'nest1': 1.0})


def test_map5():
    schema = Map(schema={'nest1': Integer(strict=False)})

    assert schema.validate({'nest1': 1.0})


def test_map6():
    schema = Map(schema={'nest1': Integer()})

    assert not schema.validate({'nest1': 1.0})


def test_map7():
    schema = Map(schema={'nest1': Integer()})

    assert not schema.validate({})


def test_map8():
    schema = Map(schema={'nest1': Integer()}, strict=False)

    assert schema.validate({})


def test_enum1():
    schema = Enum(valid_values=[])

    assert not schema.validate(1)


def test_enum2():
    schema = Enum(valid_values=[1, 2, 3])

    assert schema.validate(1)


def test_enum3():
    schema = Enum(valid_values=range(3))

    assert schema.validate(1)


def test_enum4():
    schema = Enum(valid_values=range(3))

    assert schema.validate(None)


def test_enum5():
    schema = Enum(valid_values=range(3), required=True)

    assert not schema.validate(None)


def test_enum6():
    schema = Enum(valid_values=['1', '2', '3'])

    assert not schema.validate(1)


def test_enum7():
    schema = Enum(valid_values=['1', '2', '3'], strict=False)

    assert schema.validate(1)


def test_enum8():
    schema = Enum(valid_values=['1', 'test', '3'], strict=False)

    assert schema.validate(1.0)


def test_enum9():
    schema = Enum(valid_values=[1, None, '3'], strict=False)

    assert schema.validate('1')


def test_enum10():
    schema = Enum(valid_values=['1', None, '3'])

    assert schema.validate(None)


def test_enum11():
    schema = Enum(valid_values=['1', None, '3'], required=True)

    assert not schema.validate(None)


def test_oneof1():
    schema = OneOf()

    assert not schema.validate(1)


def test_oneof2():
    schema = OneOf(valid_types=(Any(),))

    assert schema.validate(1)


def test_oneof3():
    schema = OneOf(valid_types=(Integer(), Float()))

    assert schema.validate(1)


def test_oneof4():
    schema = OneOf(valid_types=(Float(), Integer()))

    assert schema.validate(1)


def test_oneof5():
    schema = OneOf(valid_types=(Float(), String()))

    assert not schema.validate(1)



import sys

import click

from pystrictconfig.schema import schema
from pystrictconfig.utils import validate_yaml


@click.command()
@click.option('--file', prompt='Path to yaml file', help='Path to yaml file')
def main(file: str):
    click.secho(f'Hello {file}', fg='blue')
    validate_yaml(file, schema)


if __name__ == '__main__':
    print('args', sys.argv)
    main()



from pathlib import Path

import pytest

from pystrictconfig import FOLDER_DATA
from pystrictconfig import utils
from pystrictconfig.core import Integer


def test_read_yaml1():
    data = utils.validate_yaml(Path(FOLDER_DATA, 'simple_yaml_config.yaml'))
    file = {
        'rest': {
            'url': 'https://example.org/primenumbers/v1',
            'port': 8443
        },
        'prime_numbers': [
            2, 3, 5, 7, 11, 13, 17, 19
        ],
        'prime_numbers2': [
            2, 3, 5, 7, 11, 13, 17, 19
        ]
    }

    assert data == file


def test_read_yaml2():
    schema = Integer()

    with pytest.raises(ValueError):
        utils.validate_yaml(Path(FOLDER_DATA, 'basic_yaml_config.yaml'), schema)


def test_read_json1():
    data = utils.validate_json(Path(FOLDER_DATA, 'simple_json_config.json'))
    file = {
        'rest': {
            'url': 'https://example.org/primenumbers/v1',
            'port': 8443
        },
        'prime_numbers': [
            2, 3, 5, 7, 11, 13, 17, 19
        ],
        'prime_numbers2': [
            2, 3, 5, 7, 11, 13, 17, 19
        ]
    }

    assert data == file


def test_read_json2():
    schema = Integer()

    with pytest.raises(ValueError):
        utils.validate_json(Path(FOLDER_DATA, 'basic_json_config.json'), schema)



import json
from pathlib import Path

import logging
from typing import Union

import yaml

from pystrictconfig import JsonLike
from pystrictconfig.core import Any


def validate_yaml(path: Union[str, Path], schema: Any = Any()) -> JsonLike:
    """
    Read a yaml file with a schema.

    @param path: path to a yaml file
    @param schema: schema of the yaml file
    @return: the content of the file with respect to the schema
    """
    logging.debug(f'Reading yaml file: {path}')
    with open(path, 'r') as f:
        value = yaml.safe_load(f)
        if schema.validate(value):
            return schema.get(value)

        raise ValueError


def validate_json(path: Union[str, Path], schema: Any = Any()) -> JsonLike:
    """
    Read a json file with a schema.

    @param path: path to a json file
    @param schema: schema of the json file
    @return: the content of the file with respect to the schema
    """
    logging.debug(f'Reading json file: {path}')
    with open(path, 'r') as f:
        value = json.load(f)
        if schema.validate(value):
            return schema.get(value)

        raise ValueError



import pytest

from pystrictconfig.core import Any, Integer, Float, String, Bool, Invalid


def test_any1():
    schema = Any()

    assert schema.validate(1), 'Any schema should validate anything, including integers'


def test_any2():
    schema = Any()

    assert schema.validate(False), 'Any schema should validate anything, including booleans'


def test_any3():
    schema = Any()

    assert schema.validate(['test']), 'Any schema should validate anything, including lists'


def test_any4():
    schema = Any()

    assert schema.validate(None), 'Any schema should validate anything, including None'


def test_any5():
    schema = Any(required=True)

    assert not schema.validate(None), 'Any schema should validate anything, but not None if a value is required'


def test_any6():
    schema = Any(as_type=object)

    with pytest.raises(TypeError):
        schema.get(1)


def test_invalid1():
    schema = Invalid()

    assert not schema.validate(1), 'Invalid should not validate anything, including integers'


def test_invalid2():
    schema = Invalid()

    assert not schema.validate('test'), 'Invalid should not validate anything, including strings'


def test_invalid3():
    schema = Invalid()

    assert schema.get('test') == 'test', 'Invalid getter does not validate value, just getting it'


def test_integer1():
    schema = Integer()

    assert schema.validate(1), 'Integer should validate integer values'


def test_integer2():
    schema = Integer()

    assert not schema.validate(1.0), 'Integer should not validate float values'


def test_integer3():
    schema = Integer(strict=False)

    assert schema.validate(1.0), 'Integer should not validate float values, unless strict is False'


def test_integer4():
    schema = Integer()

    assert schema.get(1) == 1, 'Integer getter of a integer value just returns itself'


def test_integer5():
    schema = Integer()

    assert schema.get(1.5) == 1, 'Integer getter of a float value just cast the number to an integer'


def test_float1():
    schema = Float()

    assert schema.validate(1.0), 'Float should validate float values'


def test_float2():
    schema = Float()

    assert not schema.validate(1), 'Float should not validate integer values'


def test_float3():
    schema = Float(strict=False)

    assert schema.validate(1), 'Float should not validate integer values, unless strict is False'


def test_float4():
    schema = Float()

    assert schema.get(1.5) == 1.5, 'Float getter of a float value just returns itself'


def test_float5():
    schema = Float()

    assert schema.get(1) == 1.0, 'Float getter of an integer value just cast the number to a float'


def test_string1():
    schema = String()

    assert schema.validate('test'), 'String should validate strings'


def test_string2():
    schema = String()

    assert not schema.validate(1), 'String should not validate integers'


def test_string3():
    schema = String()

    assert schema.get(1) == '1', 'String getter of a string value just returns itself'


def test_string4():
    schema = String()

    assert schema.get(1) == '1', 'String getter of an integer value just cast the number to a string'


def test_bool1():
    schema = Bool()

    assert schema.validate(True), 'Bool should validate booleans'


def test_bool2():
    schema = Bool()

    assert schema.validate(False), 'Bool should validate booleans'


def test_bool3():
    schema = Bool()

    assert not schema.validate('SI'), 'Bool should not validate strings'


def test_bool4():
    schema = Bool(strict=False)

    assert schema.validate('SI'), 'Bool should not validate strings, unless strict is False'


def test_bool5():
    schema = Bool()

    assert schema.get('SI'), 'Bool getter checks if value is one of yes_values'


def test_bool6():
    schema = Bool()

    assert not schema.get('FALSE'), 'Bool getter checks if value is one of no_values'


def test_bool7():
    schema = Bool()

    assert schema.get('123'), 'Bool getter of any value is equal to boolean value of the value itself'


def test_bool8():
    schema = Bool()

    assert not schema.get(''), 'Bool getter of any value is equal to boolean value of the value itself'



from pystrictconfig.core import Integer, List, Map


class SumCalculator:
    def __init__(self, values: list):
        self.values = values

    def sum(self) -> int:
        return sum(self.values)


class AvgCalculator:
    def __init__(self, values: list):
        self.values = values

    def avg(self) -> float:
        return sum(self.values) / len(self.values)


class ProductCalculator:
    def __init__(self, *values: int):
        self.values = values

    def product(self) -> int:
        prod = 1
        for value in self.values:
            prod *= value

        return prod


class Name:
    def __init__(self, firstname: str, lastname: str):
        self.firstname = firstname
        self.lastname = lastname

    def name(self) -> str:
        return f'{self.firstname} {self.lastname}'

    @staticmethod
    def build(config: dict) -> 'Name':
        print(config)
        return Name(config['firstname'], config['lastname'])


def test_list1():
    schema = List(data_type=Integer(), as_type=SumCalculator)
    data = range(5)

    assert schema.get(data).sum() == 10


def test_list2():
    schema = List(as_type=SumCalculator)
    data = range(5)

    assert schema.get(data).sum() == 10


def test_list3():
    schema = List(strict=False, as_type=AvgCalculator)
    data = range(5)

    assert schema.get(data).avg() == 2.0


def test_list4():
    schema = List(strict=False, as_type=ProductCalculator, expand=True)
    data = range(5)

    assert schema.get(data).product() == 0


def test_dict1():
    schema = Map(strict=False, as_type=Name.build)
    data = {'firstname': 'first', 'lastname': 'last'}

    assert schema.get(data).name() == 'first last'


def test_dict2():
    schema = Map(strict=False, expand=True, as_type=Name)
    data = {'firstname': 'first', 'lastname': 'last'}

    assert schema.get(data).name() == 'first last'



import importlib.metadata
import logging
from pathlib import Path
from typing import Dict, Any, Callable, Type, Union

__version__ = importlib.metadata.version(__package__ or __name__)

logging.basicConfig(format='%(asctime)d-%(levelname)s-%(message)s')

# folders location

FOLDER_PACKAGE = Path(__file__).parent
FOLDER_SOURCE = FOLDER_PACKAGE.parent
FOLDER_ROOT = FOLDER_SOURCE.parent
FOLDER_DATA = Path(FOLDER_ROOT, 'data')

# custom type hints
JsonLike = Union[Dict[str, Any], Any]
TypeLike = Union[Callable, Type]



from dataclasses import dataclass
from pathlib import Path
from typing import Any as AnyValue, Iterable

from pystrictconfig import TypeLike
from pystrictconfig.core import Integer, OneOf, Any, String


@dataclass
class Port(Integer):
    def validate(self, value: AnyValue) -> bool:
        """
        Validate a value against the validator.

        @param value: value to be checked. It needs to be an integer between 0 and 65535.
        @return: True if value is compliant to validator, False otherwise
        """
        if not super().validate(value):
            return False

        value = self.get(value)

        return 0 <= value <= 65535


@dataclass
class LocalPath(OneOf):
    as_type: TypeLike = Path
    valid_types: Iterable[Any] = (String(), String(as_type=Path))
    exists: bool = True

    def validate(self, value: AnyValue) -> bool:
        """
        Validate a value against the validator.

        @param value: value to be checked. It needs to be one of the valid values.
        @return: True if value is compliant to validator, False otherwise
        """
        if not super().validate(value):
            return False

        path = self.get(value)

        if path.exists() and (self.exists is not None and not self.exists):
            return False
        if not path.exists() and self.exists:
            return False

        return True



import os
from pathlib import Path

from pystrictconfig import FOLDER_ROOT
from pystrictconfig.custom import Port, LocalPath


def test_port1():
    schema = Port()

    assert schema.validate(1234)


def test_port2():
    schema = Port()

    assert not schema.validate(123456)


def test_port3():
    schema = Port()

    assert not schema.validate('123')


def test_port4():
    schema = Port(strict=False)

    assert schema.validate('123')


def test_path1():
    schema = LocalPath()

    assert schema.validate(Path(FOLDER_ROOT, 'src'))


def test_path2():
    schema = LocalPath()

    assert schema.validate(os.path.join(FOLDER_ROOT, 'src'))


def test_path3():
    schema = LocalPath(exists=True)

    assert schema.validate(Path(FOLDER_ROOT, 'src'))


def test_path4():
    schema = LocalPath(exists=False)

    assert not schema.validate(Path(FOLDER_ROOT, 'src'))


def test_path5():
    schema = LocalPath(exists=False)

    assert schema.validate(Path(FOLDER_ROOT, 'src2'))


def test_path6():
    schema = LocalPath(exists=True)

    assert not schema.validate(Path(FOLDER_ROOT, 'src2'))


def test_path7():
    schema = LocalPath()

    assert not schema.validate(123)



import logging
from collections import defaultdict
from copy import deepcopy
from dataclasses import dataclass
from typing import Any as AnyValue, Tuple, Callable, Mapping, Sequence, Iterable

from pystrictconfig import JsonLike, TypeLike


@dataclass
class Any:
    as_type: TypeLike = None
    strict: bool = True
    required: bool = False

    def __post_init__(self):
        self._original_config = self.config

    def validate(self, value: AnyValue) -> bool:
        """
        Validate a value against the validator.

        @param value: value to be checked
        @return: True if value is compliant to validator, False otherwise
        """
        if value is None and self.required:
            logging.warning(f'{value} is None but it is required')

            return False
        # strict as config overrides value
        as_type = self.as_type or type(value)
        if self.strict and not isinstance(value, as_type):
            logging.warning(f'{value} is not of type {as_type}')

            return False

        return isinstance(self.get(value), as_type)

    def get(self, value: AnyValue) -> AnyValue:
        """
        Get the value of the type required.

        @param value: value to be gotten
        @return: the value of the required type
        @raise ValueError: if an exception occurred when creating the object
        """
        if value is None:
            return None

        if not self.as_type:
            return value

        try:
            return self.as_type(value)
        except (ValueError, TypeError) as e:
            logging.error(e)

            raise e

    @property
    def config(self) -> JsonLike:
        """
        Return configuration values of the validator.

        @return: configuration values
        """
        return deepcopy({key: value for key, value in self.__dict__.items() if not key.startswith('_')})

    @config.setter
    def config(self, config: JsonLike) -> None:
        self.__dict__.update(**config)

    def update_config(self, **config) -> 'Any':
        """
        Update default configuration values with new ones.

        @param config: any configuration as:
        - as_type
        - strict
        - required
        - anything defined in subclasses
        @return: self instance
        """
        self.config = config

        return self

    def restore_config(self) -> 'Any':
        """
        Restore configuration to default values.

        @return: self instance
        """
        self.config = self._original_config

        return self

    def clone(self) -> 'Any':
        """
        Create a copy of validator with current configuration values.

        @return: a copy of the validator
        """
        return deepcopy(self)


@dataclass
class Invalid(Any):
    def validate(self, value: AnyValue) -> bool:
        """
        An Invalid validator which does not validate any value.

        @param value: value to be checked
        @return: False
        """
        return False


@dataclass
class Integer(Any):
    as_type: TypeLike = int


@dataclass
class Float(Any):
    as_type: TypeLike = float


@dataclass
class String(Any):
    as_type: TypeLike = str


@dataclass
class Bool(Any):
    as_type: TypeLike = bool
    yes_values: Tuple[str] = ('YES', 'Y', 'SI', '1', 'TRUE')
    no_values: Tuple[str] = ('NO', 'N', '0', 'FALSE')

    def get(self, value: AnyValue) -> AnyValue:
        """
        Get the value of the type required.

        @param value: value to be gotten. It is checked against true and false values.
        @return: the value of the required type
        @raise ValueError: if an exception occurred when creating the object
        """
        value = str(value).upper()
        if value in self.yes_values:
            value = True
        elif value in self.no_values:
            value = False

        return super().get(value)


@dataclass
class List(Any):
    as_type: TypeLike = list
    data_type: TypeLike = None
    expand: bool = False

    def __post_init__(self):
        super().__post_init__()
        self.data_type = self.data_type or (Invalid() if self.strict else Any())
        if self.expand:
            self.as_type = self._builder(self.as_type)

    def validate(self, value: AnyValue) -> bool:
        """
        Validate a value against the validator.

        @param value: value to be checked. Each item of the sequence is checked against the data_type.
        @return: True if value is compliant to validator, False otherwise
        """
        if not super().validate(value):
            return False

        for el in value:
            if not self.data_type.validate(el):
                return False

        return True

    def get(self, value: Sequence) -> AnyValue:
        """
        Get the value of the type required.

        @param value: value to be gotten. Each item of the sequence is gotten with the data_type.
        @return: the value of the required type
        @raise ValueError: if an exception occurred when creating the object
        """
        return super().get([self.data_type.get(el) for el in value])

    @staticmethod
    def _builder(as_type: TypeLike) -> Callable[[list], AnyValue]:
        """
        Wrapper to type to allow star expression of value.

        @param as_type: type which require star expression
        @return: wrapper to the type
        """
        def wrapper(values: list):
            return as_type(*values)
        return wrapper


@dataclass
class Map(Any):
    as_type: TypeLike = dict
    schema: JsonLike = None
    expand: bool = False

    def __post_init__(self):
        super().__post_init__()
        default_schema = Invalid() if self.strict else Any()
        self.schema = defaultdict(lambda: default_schema, self.schema or {})
        if self.expand:
            self.as_type = self._builder(self.as_type)

    def validate(self, value: Mapping) -> bool:
        """
        Validate a value against the validator.

        @param value: value to be checked. Each value of the dictionary is checked against data_type in the schema
        @return: True if value is compliant to validator, False otherwise
        """
        if self.strict and value.keys() != self.schema.keys():
            logging.warning(f'{value.keys()} has different keys with respect to {self.schema.keys()}')

            return False

        for key, value in value.items():
            if key not in self.schema:
                logging.warning(f'{key} is missing from {self.schema.keys()}')
            if not self.schema[key].validate(value):
                logging.warning(f'{key} has an invalid value')

                return False

        return True

    def get(self, value: AnyValue) -> AnyValue:
        """
        Get the value of the type required.

        @param value: value to be gotten. Each value is gotten with the data_type in the schema
        @return: the value of the required type
        @raise ValueError: if an exception occurred when creating the object
        """
        return super().get({k: self.schema[k].get(value[k]) for k, v in value.items()})

    @staticmethod
    def _builder(as_type: TypeLike) -> Callable[[dict], AnyValue]:
        """
        Wrapper to type to allow star expression of value.

        @param as_type: type which require star expression
        @return: wrapper to the type
        """
        def wrapper(values: dict):
            return as_type(**values)
        return wrapper


@dataclass
class Enum(Any):
    valid_values: Iterable[AnyValue] = tuple()

    def __post_init__(self):
        super().__post_init__()
        if not self.valid_values:
            logging.warning('No valid value provided!')

    def validate(self, value: AnyValue) -> bool:
        """
        Validate a value against the validator.

        @param value: value to be checked. It needs to be one of the valid values.
        @return: True if value is compliant to validator, False otherwise
        """
        if not super().validate(value):
            return False

        # if here value is None then required = False
        if value is not None:
            data_type = type(value)
            for el in self.valid_values:
                if (value == el and (type(value) == type(el))) or (not self.strict and value == data_type(el)):
                    break
            else:
                logging.warning(f'{value} is not one of {self.valid_values}')

                return False

        return True


@dataclass
class OneOf(Any):
    valid_types: Iterable[Any] = tuple()

    def __post_init__(self):
        super().__post_init__()
        if not self.valid_types:
            logging.warning('No valid type provided!')

    def validate(self, value: AnyValue) -> bool:
        """
        Validate a value against the validator.

        @param value: value to be checked. It needs to be one of the valid values.
        @return: True if value is compliant to validator, False otherwise
        """
        for valid_type in self.valid_types:
            if valid_type.validate(value):
                break
        else:
            logging.warning(f'{value} is not one of {self.valid_types}')

            return False

        return True



from pystrictconfig.core import Any, Map


def test_update1():
    schema = Any(required=True)

    assert schema.update_config(required=False).validate(None), 'Configuration values can be updated by update_config'


def test_update2():
    schema = Any(required=True)

    assert not schema.update_config(required=False).restore_config().validate(None), \
        'Configuration values can be updated by update_config and restored by restore_config'


def test_clone1():
    schema = Any()

    assert schema == schema.clone(), 'A schema should be equal to a copy of itself'


def test_clone2():
    schema = Any()

    assert not schema == schema.clone().update_config(required=True), 'An update of a schema does not afflict copies'


def test_clone3():
    schema = Map(schema={'test1': Any()})

    assert not schema == schema.clone().update_config(schema={'test2': Any()}), \
        'A deepcopy of the schema is executed, not a shallow copy'



from pystrictconfig.core import Integer, Float, String, Bool, Map

schema = Map(schema={
    'integer': Integer(),
    'float1': Float(strict=False),
    'float2': Float(),
    'string': String(),
    'bool1': Bool(),
    'bool2': Bool(),
    'bool3': Bool(),
    'bool4': Bool(),
    'bool5': Bool(),
    'bool6': Bool(),
    'bool7': Bool(),
})
